@@ -134,7 +134,7 @@ $ databroker-pack CATALOG --all --copy-external DIRECTORY
         action="store_true",
         help=(
             "By default, the locations of all relevant external files on the "
-            f"source machine are written to text files. "
+            "source machine are written to text files. "
             "Set this to omit those manifests."
         ),
     )
@@ -152,6 +152,14 @@ $ databroker-pack CATALOG --all --copy-external DIRECTORY
     other_group.add_argument(
         "--no-documents", action="store_true", help="Do not pack the Documents.",
     )
+    other_group.add_argument(
+        "--limit",
+        type=int,
+        help=(
+            "Stop after exporting some number of Runs. Useful for testing a "
+            "subset before doing a lengthy export."
+        ),
+    )
     other_group.add_argument(
         "--strict",
         action="store_true",
@@ -169,6 +177,11 @@ $ databroker-pack CATALOG --all --copy-external DIRECTORY
             "If unspecified, automatic handler discovery is used."
         ),
     )
+    other_group.add_argument(
+        "--salt",
+        type=str.encode,  # casts input to bytes
+        help=("Set this to override the random default with a fixed value."),
+    )
     other_group.add_argument(
         "-V",
         "--version",
@@ -274,14 +287,16 @@ $ databroker-pack CATALOG --all --copy-external DIRECTORY
             if not results:
                 print(f"Query {combined_query} yielded no results. Exiting.")
                 sys.exit(1)
-            artifacts, external_files, failures = export_catalog(
+            artifacts, external_files, failures, file_uids = export_catalog(
                 results,
                 manager,
                 strict=args.strict,
                 external=external,
-                dry_run=args.no_documents,
+                no_documents=args.no_documents,
                 handler_registry=handler_registry,
                 serializer_class=serializer_class,
+                salt=args.salt,
+                limit=args.limit,
             )
         elif args.uids:
             # Skip blank lines and commented lines.
@@ -295,16 +310,17 @@ $ databroker-pack CATALOG --all --copy-external DIRECTORY
             if not uids:
                 print("Found empty input for --uids. Exiting")
                 sys.exit(1)
-                artifacts, external_files, failures = export_uids(
-                    catalog,
-                    uids,
-                    manager,
-                    strict=args.strict,
-                    external=external,
-                    dry_run=args.no_documents,
-                    handler_registry=handler_registry,
-                    serializer_class=serializer_class,
-                )
+            artifacts, external_files, failures, file_uids = export_uids(
+                catalog,
+                uids[:args.limit],
+                manager,
+                strict=args.strict,
+                external=external,
+                no_documents=args.no_documents,
+                handler_registry=handler_registry,
+                serializer_class=serializer_class,
+                salt=args.salt,
+            )
         else:
             parser.error(
                 "Must specify which Runs to pack, --query ... or "
@@ -321,28 +337,52 @@ $ databroker-pack CATALOG --all --copy-external DIRECTORY
             # reference its location.
             if args.copy_external:
                 target_drectory = pathlib.Path(args.directory, "external_files")
-                for root, files in external_files.items():
+                for (
+                    (root_in_document, root, unique_id),
+                    files,
+                ) in external_files.items():
                     new_root, new_files, copying_failures_ = copy_external_files(
-                        target_drectory, root, files, strict=args.strict
+                        target_drectory, root, unique_id, files, strict=args.strict
                     )
                     copying_failures.extend(copying_failures_)
-                    # Record the root relative to the pack directory.
+                    # The root_map value will be the relative path to
+                    # the data within the pack directory.
                     relative_root = new_root.relative_to(args.directory)
-                    root_map.update({root: relative_root})
+                    if not args.no_documents:
+                        # When we are exporting documents, we rewrite the
+                        # 'root' key in the Resource to unique_id to ensure
+                        # no collisions of the keys in root_map.
+                        root_map.update({unique_id: relative_root})
+                    else:
+                        # If we are not exporting documents, the root_map has
+                        # to refer to the root as it is.
+                        root_map.update({root_in_document: relative_root})
                     rel_paths = [
                         pathlib.Path(f).relative_to(args.directory) for f in new_files
                     ]
-                    write_external_files_manifest(manager, root, rel_paths)
+                    write_external_files_manifest(manager, unique_id, rel_paths)
             else:
-                for root, files in external_files.items():
-                    # Make root_map an identity map.
-                    root_map.update({root: root})
-                    write_external_files_manifest(manager, root, files)
+                for (
+                    (root_in_document, root, unique_id),
+                    files,
+                ) in external_files.items():
+                    # The root_map value will be the current absolute path to
+                    # the data.
+                    if not args.no_documents:
+                        # When we are exporting documents, we rewrite the
+                        # 'root' key in the Resource to unique_id to ensure
+                        # no collisions of the keys in root_map.
+                        root_map.update({unique_id: root})
+                    else:
+                        # If we are not exporting documents, the root_map has
+                        # to refer to the root as it is.
+                        root_map.update({root_in_document: root})
+                    write_external_files_manifest(manager, unique_id, files)
         if args.format == "jsonl":
-            paths = ["./*.jsonl"]
+            paths = ["./documents/*.jsonl"]
             write_jsonl_catalog_file(manager, args.directory, paths, root_map)
         elif args.format == "msgpack":
-            paths = ["./*.msgpack"]
+            paths = ["./documents/*.msgpack"]
             write_msgpack_catalog_file(manager, args.directory, paths, root_map)
         # No need for an else here; we validated that it is one of these above.
         if failures:
